@@ -18,7 +18,6 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -43,9 +42,9 @@ public class AspectJElementHierarchy implements IHierarchy {
 
 	// Access to the handleMap and typeMap are now synchronized - at least the find methods and the updateHandleMap function
 	// see pr305788
-	private Map fileMap = null;
-	private Map handleMap = new HashMap();
-	private Map typeMap = null;
+	private Map<String, IProgramElement> fileMap = null;
+	private Map<String, IProgramElement> handleMap = new HashMap<String, IProgramElement>();
+	private Map<String, IProgramElement> typeMap = null;
 
 	public AspectJElementHierarchy(AsmManager asm) {
 		this.asm = asm;
@@ -63,24 +62,41 @@ public class AspectJElementHierarchy implements IHierarchy {
 		return root;
 	}
 
-	public void setRoot(IProgramElement root) {
-		this.root = root;
-		handleMap = new HashMap();
-		typeMap = new HashMap();
+	public String toSummaryString() {
+		StringBuilder s = new StringBuilder();
+		s.append("FileMap has " + fileMap.size() + " entries\n");
+		s.append("HandleMap has " + handleMap.size() + " entries\n");
+		s.append("TypeMap has " + handleMap.size() + " entries\n");
+		s.append("FileMap:\n");
+		for (Map.Entry<String, IProgramElement> fileMapEntry : fileMap.entrySet()) {
+			s.append(fileMapEntry).append("\n");
+		}
+		s.append("TypeMap:\n");
+		for (Map.Entry<String, IProgramElement> typeMapEntry : typeMap.entrySet()) {
+			s.append(typeMapEntry).append("\n");
+		}
+		s.append("HandleMap:\n");
+		for (Map.Entry<String, IProgramElement> handleMapEntry : handleMap.entrySet()) {
+			s.append(handleMapEntry).append("\n");
+		}
+		return s.toString();
 	}
 
-	public void addToFileMap(Object key, Object value) {
+	public void setRoot(IProgramElement root) {
+		this.root = root;
+		handleMap = new HashMap<String, IProgramElement>();
+		typeMap = new HashMap<String, IProgramElement>();
+	}
+
+	public void addToFileMap(String key, IProgramElement value) {
 		fileMap.put(key, value);
 	}
 
-	public boolean removeFromFileMap(Object key) {
-		if (fileMap.containsKey(key)) {
-			return (fileMap.remove(key) != null);
-		}
-		return true;
+	public boolean removeFromFileMap(String canonicalFilePath) {
+		return fileMap.remove(canonicalFilePath) != null;
 	}
 
-	public void setFileMap(HashMap fileMap) {
+	public void setFileMap(HashMap<String, IProgramElement> fileMap) {
 		this.fileMap = fileMap;
 	}
 
@@ -88,7 +104,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 		return fileMap.get(key);
 	}
 
-	public Set getFileMapEntrySet() {
+	public Set<Map.Entry<String, IProgramElement>> getFileMapEntrySet() {
 		return fileMap.entrySet();
 	}
 
@@ -104,8 +120,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 	 * @return null if not found
 	 */
 	public IProgramElement findElementForSignature(IProgramElement parent, IProgramElement.Kind kind, String signature) {
-		for (Iterator it = parent.getChildren().iterator(); it.hasNext();) {
-			IProgramElement node = (IProgramElement) it.next();
+		for (IProgramElement node : parent.getChildren()) {
 			if (node.getKind() == kind && signature.equals(node.toSignatureString())) {
 				return node;
 			} else {
@@ -119,9 +134,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 	}
 
 	public IProgramElement findElementForLabel(IProgramElement parent, IProgramElement.Kind kind, String label) {
-
-		for (Iterator it = parent.getChildren().iterator(); it.hasNext();) {
-			IProgramElement node = (IProgramElement) it.next();
+		for (IProgramElement node : parent.getChildren()) {
 			if (node.getKind() == kind && label.equals(node.toLabelString())) {
 				return node;
 			} else {
@@ -148,18 +161,16 @@ public class AspectJElementHierarchy implements IHierarchy {
 			StringBuffer keyb = (packageName == null) ? new StringBuffer() : new StringBuffer(packageName);
 			keyb.append(".").append(typeName);
 			String key = keyb.toString();
-			IProgramElement cachedValue = (IProgramElement) typeMap.get(key);
+			IProgramElement cachedValue = typeMap.get(key);
 			if (cachedValue != null) {
 				return cachedValue;
 			}
 
-			List packageNodes = findMatchingPackages(packageName);
+			List<IProgramElement> packageNodes = findMatchingPackages(packageName);
 
-			for (Iterator iterator = packageNodes.iterator(); iterator.hasNext();) {
-				IProgramElement pkg = (IProgramElement) iterator.next();
+			for (IProgramElement pkg : packageNodes) {
 				// this searches each file for a class
-				for (Iterator it = pkg.getChildren().iterator(); it.hasNext();) {
-					IProgramElement fileNode = (IProgramElement) it.next();
+				for (IProgramElement fileNode : pkg.getChildren()) {
 					IProgramElement cNode = findClassInNodes(fileNode.getChildren(), typeName, typeName);
 					if (cNode != null) {
 						typeMap.put(key, cNode);
@@ -210,21 +221,19 @@ public class AspectJElementHierarchy implements IHierarchy {
 	 * @param packagename the packagename being searched for
 	 * @return a list of package nodes that match that name
 	 */
-	public List/* IProgramElement */findMatchingPackages(String packagename) {
-		List children = root.getChildren();
+	public List<IProgramElement> findMatchingPackages(String packagename) {
+		List<IProgramElement> children = root.getChildren();
 		// The children might be source folders or packages
 		if (children.size() == 0) {
-			return Collections.EMPTY_LIST;
+			return Collections.emptyList();
 		}
-		if (((IProgramElement) children.get(0)).getKind() == IProgramElement.Kind.SOURCE_FOLDER) {
+		if ((children.get(0)).getKind() == IProgramElement.Kind.SOURCE_FOLDER) {
 			String searchPackageName = (packagename == null ? "" : packagename); // default package means match on ""
 			// dealing with source folders
-			List matchingPackageNodes = new ArrayList();
-			for (Iterator iterator = children.iterator(); iterator.hasNext();) {
-				IProgramElement sourceFolder = (IProgramElement) iterator.next();
-				List possiblePackageNodes = sourceFolder.getChildren();
-				for (Iterator iterator2 = possiblePackageNodes.iterator(); iterator2.hasNext();) {
-					IProgramElement possiblePackageNode = (IProgramElement) iterator2.next();
+			List<IProgramElement> matchingPackageNodes = new ArrayList<IProgramElement>();
+			for (IProgramElement sourceFolder : children) {
+				List<IProgramElement> possiblePackageNodes = sourceFolder.getChildren();
+				for (IProgramElement possiblePackageNode : possiblePackageNodes) {
 					if (possiblePackageNode.getKind() == IProgramElement.Kind.PACKAGE) {
 						if (possiblePackageNode.getName().equals(searchPackageName)) {
 							matchingPackageNodes.add(possiblePackageNode);
@@ -239,20 +248,18 @@ public class AspectJElementHierarchy implements IHierarchy {
 			// thing to return in the list
 			if (packagename == null) {
 				// default package
-				List result = new ArrayList();
+				List<IProgramElement> result = new ArrayList<IProgramElement>();
 				result.add(root);
 				return result;
 			}
-			List result = new ArrayList();
-			for (Iterator iterator = children.iterator(); iterator.hasNext();) {
-				IProgramElement possiblePackage = (IProgramElement) iterator.next();
+			List<IProgramElement> result = new ArrayList<IProgramElement>();
+			for (IProgramElement possiblePackage : children) {
 				if (possiblePackage.getKind() == IProgramElement.Kind.PACKAGE && possiblePackage.getName().equals(packagename)) {
 					result.add(possiblePackage);
 				}
 				if (possiblePackage.getKind() == IProgramElement.Kind.SOURCE_FOLDER) { // might be 'binaries'
 					if (possiblePackage.getName().equals("binaries")) {
-						for (Iterator iter2 = possiblePackage.getChildren().iterator(); iter2.hasNext();) {
-							IProgramElement possiblePackage2 = (IProgramElement) iter2.next();
+						for (IProgramElement possiblePackage2 : possiblePackage.getChildren()) {
 							if (possiblePackage2.getKind() == IProgramElement.Kind.PACKAGE
 									&& possiblePackage2.getName().equals(packagename)) {
 								result.add(possiblePackage2);
@@ -263,14 +270,14 @@ public class AspectJElementHierarchy implements IHierarchy {
 				}
 			}
 			if (result.isEmpty()) {
-				return Collections.EMPTY_LIST;
+				return Collections.emptyList();
 			} else {
 				return result;
 			}
 		}
 	}
 
-	private IProgramElement findClassInNodes(Collection nodes, String name, String typeName) {
+	private IProgramElement findClassInNodes(Collection<IProgramElement> nodes, String name, String typeName) {
 		String baseName;
 		String innerName;
 		int dollar = name.indexOf('$');
@@ -282,8 +289,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 			innerName = name.substring(dollar + 1);
 		}
 
-		for (Iterator j = nodes.iterator(); j.hasNext();) {
-			IProgramElement classNode = (IProgramElement) j.next();
+		for (IProgramElement classNode : nodes) {
 			if (baseName.equals(classNode.getName())) {
 				if (innerName == null) {
 					return classNode;
@@ -383,8 +389,8 @@ public class AspectJElementHierarchy implements IHierarchy {
 			return null; // no need to search children of a source file node
 		} else {
 			// check the children
-			for (Iterator iterator = node.getChildren().iterator(); iterator.hasNext();) {
-				IProgramElement foundit = findNodeForSourceFile((IProgramElement) iterator.next(), sourcefilePath);
+			for (IProgramElement child : node.getChildren()) {
+				IProgramElement foundit = findNodeForSourceFile(child, sourcefilePath);
 				if (foundit != null) {
 					return foundit;
 				}
@@ -435,8 +441,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 		if (node == null || node.getChildren() == null) {
 			return null;
 		}
-		for (Iterator childrenIter = node.getChildren().iterator(); childrenIter.hasNext();) {
-			IProgramElement child = (IProgramElement) childrenIter.next();
+		for (IProgramElement child : node.getChildren()) {
 			ISourceLocation childLoc = child.getSourceLocation();
 			if (childLoc != null) {
 				if (childLoc.getLine() <= lineno && childLoc.getEndLine() >= lineno) {
@@ -465,8 +470,8 @@ public class AspectJElementHierarchy implements IHierarchy {
 		}
 
 		if (node != null) {
-			for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
-				IProgramElement foundNode = findNodeForSourceLineHelper((IProgramElement) it.next(), sourceFilePath, lineno, offset);
+			for (IProgramElement child : node.getChildren()) {
+				IProgramElement foundNode = findNodeForSourceLineHelper(child, sourceFilePath, lineno, offset);
 				if (foundNode != null) {
 					return foundNode;
 				}
@@ -496,8 +501,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 	}
 
 	private boolean hasMoreSpecificChild(IProgramElement node, String sourceFilePath, int lineNumber, int offSet) {
-		for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
-			IProgramElement child = (IProgramElement) it.next();
+		for (IProgramElement child : node.getChildren()) {
 			if (matches(child, sourceFilePath, lineNumber, offSet)) {
 				return true;
 			}
@@ -524,11 +528,10 @@ public class AspectJElementHierarchy implements IHierarchy {
 		// try the cache first...
 		IProgramElement ipe = null;
 		synchronized (this) {
-			ipe = (IProgramElement) handleMap.get(handle);
+			ipe = handleMap.get(handle);
 			if (ipe != null) {
 				return ipe;
 			}
-
 			ipe = findElementForHandle(root, handle);
 			if (ipe == null && create) {
 				ipe = createFileStructureNode(getFilename(handle));
@@ -541,8 +544,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 	}
 
 	private IProgramElement findElementForHandle(IProgramElement parent, String handle) {
-		for (Iterator it = parent.getChildren().iterator(); it.hasNext();) {
-			IProgramElement node = (IProgramElement) it.next();
+		for (IProgramElement node : parent.getChildren()) {
 			String nodeHid = node.getHandleIdentifier();
 			if (handle.equals(nodeHid)) {
 				return node;
@@ -559,7 +561,7 @@ public class AspectJElementHierarchy implements IHierarchy {
 		return null;
 	}
 
-	//	
+	//
 	// private IProgramElement findElementForBytecodeInfo(
 	// IProgramElement node,
 	// String parentName,
@@ -598,49 +600,75 @@ public class AspectJElementHierarchy implements IHierarchy {
 		fileMap.clear();
 	}
 
+	public void forget(IProgramElement compilationUnitNode, IProgramElement typeNode) {
+		String k = null;
+		synchronized (this) {
+			// handle map
+			// type map
+			for (Map.Entry<String, IProgramElement> typeMapEntry : typeMap.entrySet()) {
+				if (typeMapEntry.getValue() == typeNode) {
+					k = typeMapEntry.getKey();
+					break;
+				}
+			}
+			if (k != null) {
+				typeMap.remove(k);
+			}
+		}
+
+		if (compilationUnitNode != null) {
+			k = null;
+			for (Map.Entry<String, IProgramElement> entry : fileMap.entrySet()) {
+				if (entry.getValue() == compilationUnitNode) {
+					k = entry.getKey();
+					break;
+				}
+			}
+			if (k != null) {
+				fileMap.remove(k);
+			}
+		}
+	}
+
 	// TODO rename this method ... it does more than just the handle map
-	public void updateHandleMap(Set deletedFiles) {
+	public void updateHandleMap(Set<String> deletedFiles) {
 		// Only delete the entries we need to from the handle map - for performance reasons
-		List forRemoval = new ArrayList();
-		Set k = null;
+		List<String> forRemoval = new ArrayList<String>();
+		Set<String> k = null;
 		synchronized (this) {
 			k = handleMap.keySet();
-			for (Iterator iter = k.iterator(); iter.hasNext();) {
-				String handle = (String) iter.next();
-				IProgramElement ipe = (IProgramElement) handleMap.get(handle);
-				if (deletedFiles.contains(getCanonicalFilePath(ipe))) {
+			for (String handle : k) {
+				IProgramElement ipe = handleMap.get(handle);
+				if (ipe == null) {
+					System.err.println("handleMap expectation not met, where is the IPE for " + handle);
+				}
+				if (ipe == null || deletedFiles.contains(getCanonicalFilePath(ipe))) {
 					forRemoval.add(handle);
 				}
 			}
-			for (Iterator iter = forRemoval.iterator(); iter.hasNext();) {
-				String handle = (String) iter.next();
+			for (String handle : forRemoval) {
 				handleMap.remove(handle);
 			}
 			forRemoval.clear();
 			k = typeMap.keySet();
-			for (Iterator iter = k.iterator(); iter.hasNext();) {
-				String typeName = (String) iter.next();
-				IProgramElement ipe = (IProgramElement) typeMap.get(typeName);
+			for (String typeName : k) {
+				IProgramElement ipe = typeMap.get(typeName);
 				if (deletedFiles.contains(getCanonicalFilePath(ipe))) {
 					forRemoval.add(typeName);
 				}
 			}
-			for (Iterator iter = forRemoval.iterator(); iter.hasNext();) {
-				String typeName = (String) iter.next();
+			for (String typeName : forRemoval) {
 				typeMap.remove(typeName);
 			}
 			forRemoval.clear();
 		}
-		k = fileMap.keySet();
-		for (Iterator iter = k.iterator(); iter.hasNext();) {
-			String filePath = (String) iter.next();
-			IProgramElement ipe = (IProgramElement) fileMap.get(filePath);
-			if (deletedFiles.contains(getCanonicalFilePath(ipe))) {
+		for (Map.Entry<String, IProgramElement> entry : fileMap.entrySet()) {
+			String filePath = entry.getKey();
+			if (deletedFiles.contains(getCanonicalFilePath(entry.getValue()))) {
 				forRemoval.add(filePath);
 			}
 		}
-		for (Iterator iter = forRemoval.iterator(); iter.hasNext();) {
-			String filePath = (String) iter.next();
+		for (String filePath : forRemoval) {
 			fileMap.remove(filePath);
 		}
 	}
