@@ -555,7 +555,7 @@ public class AspectDeclaration extends TypeDeclaration {
 				BranchLabel instanceFound = new BranchLabel(codeStream);
 
 				ExceptionLabel anythingGoesWrong = new ExceptionLabel(codeStream, world
-						.makeTypeBinding(UnresolvedType.JAVA_LANG_EXCEPTION));
+						.makeTypeBinding(UnresolvedType.JL_EXCEPTION));
 				anythingGoesWrong.placeStart();
 				codeStream.aload_0();
 				codeStream.invokestatic(world.makeMethodBindingForCall(AjcMemberMaker.perTypeWithinGetInstance(typeX)));
@@ -650,7 +650,7 @@ public class AspectDeclaration extends TypeDeclaration {
 		final EclipseFactory world = EclipseFactory.fromScopeLookupEnvironment(this.scope);
 		generateMethod(classFile, hasAspectMethod, new BodyGenerator() {
 			public void generate(CodeStream codeStream) {
-				ExceptionLabel goneBang = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JAVA_LANG_EXCEPTION));
+				ExceptionLabel goneBang = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JL_EXCEPTION));
 				BranchLabel noInstanceExists = new BranchLabel(codeStream);
 				BranchLabel leave = new BranchLabel(codeStream);
 				goneBang.placeStart();
@@ -708,16 +708,17 @@ public class AspectDeclaration extends TypeDeclaration {
 		/*
 		 * Code: Stack=1, Locals=1, Args_size=1 0: aload_0 1: getfield #14; //Field ajc$withinType:Ljava/lang/String; 4: areturn
 		 */
-		generateMethod(classFile, AjcMemberMaker.perTypeWithinGetWithinTypeNameMethod(world.fromBinding(binding), world.getWorld()
-				.isInJava5Mode()), new BodyGenerator() {
-			public void generate(CodeStream codeStream) {
-				ExceptionLabel exc = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JAVA_LANG_EXCEPTION));
-				exc.placeStart();
-				codeStream.aload_0();
-				codeStream.getfield(world.makeFieldBinding(AjcMemberMaker.perTypeWithinWithinTypeField(typeX, typeX)));
-				codeStream.areturn();
-			}
-		});
+		generateMethod(classFile,
+				AjcMemberMaker.perTypeWithinGetWithinTypeNameMethod(world.fromBinding(binding), world.getWorld().isInJava5Mode()),
+				new BodyGenerator() {
+					public void generate(CodeStream codeStream) {
+						ExceptionLabel exc = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JL_EXCEPTION));
+						exc.placeStart();
+						codeStream.aload_0();
+						codeStream.getfield(world.makeFieldBinding(AjcMemberMaker.perTypeWithinWithinTypeField(typeX, typeX)));
+						codeStream.areturn();
+					}
+				});
 	}
 
 	// PTWIMPL Generate getInstance method
@@ -725,7 +726,7 @@ public class AspectDeclaration extends TypeDeclaration {
 		final EclipseFactory world = EclipseFactory.fromScopeLookupEnvironment(this.scope);
 		generateMethod(classFile, AjcMemberMaker.perTypeWithinGetInstance(world.fromBinding(binding)), new BodyGenerator() {
 			public void generate(CodeStream codeStream) {
-				ExceptionLabel exc = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JAVA_LANG_EXCEPTION));
+				ExceptionLabel exc = new ExceptionLabel(codeStream, world.makeTypeBinding(UnresolvedType.JL_EXCEPTION));
 				exc.placeStart();
 				codeStream.aload_0();
 				codeStream.ldc(NameMangler.perTypeWithinLocalAspectOf(typeX));
@@ -734,17 +735,15 @@ public class AspectDeclaration extends TypeDeclaration {
 						world.makeTypeBinding(UnresolvedType.forSignature("Ljava/lang/reflect/Method;")), // return type
 						new TypeBinding[] { world.makeTypeBinding(UnresolvedType.forSignature("Ljava/lang/String;")),
 								world.makeTypeBinding(UnresolvedType.forSignature("[Ljava/lang/Class;")) },
-						new ReferenceBinding[0], (ReferenceBinding) world.makeTypeBinding(UnresolvedType.JAVA_LANG_CLASS)));
+						new ReferenceBinding[0], (ReferenceBinding) world.makeTypeBinding(UnresolvedType.JL_CLASS)));
 				codeStream.astore_1();
 				codeStream.aload_1();
 				codeStream.aconst_null();
 				codeStream.aconst_null();
-				codeStream
-						.invokevirtual(new MethodBinding(0, "invoke".toCharArray(), world.makeTypeBinding(UnresolvedType.OBJECT),
-								new TypeBinding[] { world.makeTypeBinding(UnresolvedType.OBJECT),
-										world.makeTypeBinding(UnresolvedType.forSignature("[Ljava/lang/Object;")) },
-								new ReferenceBinding[0], (ReferenceBinding) world
-										.makeTypeBinding(UnresolvedType.JAVA_LANG_REFLECT_METHOD)));
+				codeStream.invokevirtual(new MethodBinding(0, "invoke".toCharArray(), world.makeTypeBinding(UnresolvedType.OBJECT),
+						new TypeBinding[] { world.makeTypeBinding(UnresolvedType.OBJECT),
+								world.makeTypeBinding(UnresolvedType.forSignature("[Ljava/lang/Object;")) },
+						new ReferenceBinding[0], (ReferenceBinding) world.makeTypeBinding(UnresolvedType.JAVA_LANG_REFLECT_METHOD)));
 				codeStream.checkcast(world.makeTypeBinding(typeX));
 				codeStream.astore_2();
 				codeStream.aload_2();
@@ -792,7 +791,7 @@ public class AspectDeclaration extends TypeDeclaration {
 				// codeStream.ifnull(isNull);
 				// codeStream.areturn();
 				// isNull.place();
-				//				
+				//
 				// codeStream.incrStackSize(+1); // the dup trick above confuses the stack counter
 				// codeStream.new_(world.makeTypeBinding(AjcMemberMaker.NO_ASPECT_BOUND_EXCEPTION));
 				// codeStream.dup();
@@ -1086,7 +1085,7 @@ public class AspectDeclaration extends TypeDeclaration {
 	// public String toStringBody(int tab) {
 	//
 	//		String s = " {"; //$NON-NLS-1$
-	//		
+	//
 	//
 	// if (memberTypes != null) {
 	// for (int i = 0; i < memberTypes.length; i++) {
@@ -1146,16 +1145,26 @@ public class AspectDeclaration extends TypeDeclaration {
 		// TODO deal with itd of it onto an interface
 
 		SourceTypeBinding targetSourceTypeBinding = (SourceTypeBinding) world.makeTypeBinding(munger.getTargetType());
-		ReferenceBinding[] existingMemberTypes = targetSourceTypeBinding.memberTypes();
-		for (int i = 0; i < existingMemberTypes.length; i++) {
-			char[] compounded = CharOperation.concatWith(existingMemberTypes[i].compoundName, '.');
-			if (CharOperation.endsWith(compounded, mungerMemberTypeName)) {
-				scope.problemReporter().signalError(sourceStart(), sourceEnd(),
-						"target type already declares a member type with the name '" + munger.getMemberTypeName() + "'");
-				return;
+
+		// if it is a binary type binding it is likely to be something we ITD'd on before
+		// TODO should probably avoid putting it onto BTBs at all (since already there)
+		if (!(targetSourceTypeBinding instanceof BinaryTypeBinding)) {
+			ReferenceBinding[] existingMemberTypes = targetSourceTypeBinding.memberTypes();
+			for (int i = 0; i < existingMemberTypes.length; i++) {
+				char[] compounded = CharOperation.concatWith(existingMemberTypes[i].compoundName, '.');
+				if (CharOperation.endsWith(compounded, mungerMemberTypeName)) {
+					scope.problemReporter().signalError(sourceStart(), sourceEnd(),
+							"target type already declares a member type with the name '" + munger.getMemberTypeName() + "'");
+					return;
+				}
 			}
 		}
-
+		/*
+		 * char[][] className = CharOperation.deepCopy(targetSourceTypeBinding.compoundName); className[className.length - 1] =
+		 * CharOperation.concat(className[className.length - 1], munger.getMemberTypeName() .toCharArray(), '$'); //
+		 * ReferenceBinding existingType = packageBinding.getType0(className[className.length - 1]); innerTypeBinding.compoundName =
+		 * className; innerTypeBinding.fPackage = targetSourceTypeBinding.fPackage;
+		 */
 		findOrCreateInterTypeMemberClassFinder(targetSourceTypeBinding).addInterTypeMemberType(innerTypeBinding);
 	}
 
