@@ -287,7 +287,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 				// System.err.println("XXXX start inc ");
 				AsmManager.setLastActiveStructureModel(state.getStructureModel());
 				binarySourcesForTheNextCompile = state.getBinaryFilesToCompile(true);
-				Set files = state.getFilesToCompile(true);
+				Set<File> files = state.getFilesToCompile(true);
 				if (buildConfig.isEmacsSymMode() || buildConfig.isGenerateModelMode()) {
 					if (AsmManager.attemptIncrementalModelRepairs) {
 						state.getStructureModel().resetDeltaProcessing();
@@ -462,7 +462,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		if (buildConfig.getSourcePathResources() != null) {
 			for (Iterator i = buildConfig.getSourcePathResources().keySet().iterator(); i.hasNext();) {
 				String resource = (String) i.next();
-				File from = (File) buildConfig.getSourcePathResources().get(resource);
+				File from = buildConfig.getSourcePathResources().get(resource);
 				copyResourcesFromFile(from, resource, from);
 			}
 		}
@@ -649,12 +649,12 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		String filename = buildConfig.getOutxmlName();
 		// System.err.println("? AjBuildManager.writeOutxmlFile() outxml=" + filename);
 
-		Map outputDirsAndAspects = findOutputDirsForAspects();
-		Set outputDirs = outputDirsAndAspects.entrySet();
-		for (Iterator iterator = outputDirs.iterator(); iterator.hasNext();) {
-			Map.Entry entry = (Map.Entry) iterator.next();
-			File outputDir = (File) entry.getKey();
-			List aspects = (List) entry.getValue();
+		Map<File, List<String>> outputDirsAndAspects = findOutputDirsForAspects();
+		Set<Map.Entry<File, List<String>>> outputDirs = outputDirsAndAspects.entrySet();
+		for (Iterator<Map.Entry<File, List<String>>> iterator = outputDirs.iterator(); iterator.hasNext();) {
+			Map.Entry<File, List<String>> entry = iterator.next();
+			File outputDir = entry.getKey();
+			List<String> aspects = entry.getValue();
 			ByteArrayOutputStream baos = getOutxmlContents(aspects);
 			if (zos != null) {
 				ZipEntry newEntry = new ZipEntry(filename);
@@ -697,9 +697,9 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	 * Returns a map where the keys are File objects corresponding to all the output directories and the values are a list of
 	 * aspects which are sent to that ouptut directory
 	 */
-	private Map /* File --> List (String) */findOutputDirsForAspects() {
-		Map outputDirsToAspects = new HashMap();
-		Map aspectNamesToFileNames = state.getAspectNamesToFileNameMap();
+	private Map<File, List<String>> findOutputDirsForAspects() {
+		Map<File, List<String>> outputDirsToAspects = new HashMap<File, List<String>>();
+		Map<String, char[]> aspectNamesToFileNames = state.getAspectNamesToFileNameMap();
 		if (buildConfig.getCompilationResultDestinationManager() == null
 				|| buildConfig.getCompilationResultDestinationManager().getAllOutputLocations().size() == 1) {
 			// we only have one output directory...which simplifies things
@@ -707,11 +707,10 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 			if (buildConfig.getCompilationResultDestinationManager() != null) {
 				outputDir = buildConfig.getCompilationResultDestinationManager().getDefaultOutputLocation();
 			}
-			List aspectNames = new ArrayList();
+			List<String> aspectNames = new ArrayList<String>();
 			if (aspectNamesToFileNames != null) {
-				Set keys = aspectNamesToFileNames.keySet();
-				for (Iterator iterator = keys.iterator(); iterator.hasNext();) {
-					String name = (String) iterator.next();
+				Set<String> keys = aspectNamesToFileNames.keySet();
+				for (String name : keys) {
 					aspectNames.add(name);
 				}
 			}
@@ -720,19 +719,21 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 			List outputDirs = buildConfig.getCompilationResultDestinationManager().getAllOutputLocations();
 			for (Iterator iterator = outputDirs.iterator(); iterator.hasNext();) {
 				File outputDir = (File) iterator.next();
-				outputDirsToAspects.put(outputDir, new ArrayList());
+				outputDirsToAspects.put(outputDir, new ArrayList<String>());
 			}
-			Set entrySet = aspectNamesToFileNames.entrySet();
-			for (Iterator iterator = entrySet.iterator(); iterator.hasNext();) {
-				Map.Entry entry = (Map.Entry) iterator.next();
-				String aspectName = (String) entry.getKey();
-				char[] fileName = (char[]) entry.getValue();
-				File outputDir = buildConfig.getCompilationResultDestinationManager().getOutputLocationForClass(
-						new File(new String(fileName)));
-				if (!outputDirsToAspects.containsKey(outputDir)) {
-					outputDirsToAspects.put(outputDir, new ArrayList());
+			if (aspectNamesToFileNames != null) {
+				Set<Map.Entry<String, char[]>> entrySet = aspectNamesToFileNames.entrySet();
+				for (Iterator<Map.Entry<String, char[]>> iterator = entrySet.iterator(); iterator.hasNext();) {
+					Map.Entry<String, char[]> entry = iterator.next();
+					String aspectName = entry.getKey();
+					char[] fileName = entry.getValue();
+					File outputDir = buildConfig.getCompilationResultDestinationManager().getOutputLocationForClass(
+							new File(new String(fileName)));
+					if (!outputDirsToAspects.containsKey(outputDir)) {
+						outputDirsToAspects.put(outputDir, new ArrayList<String>());
+					}
+					((List) outputDirsToAspects.get(outputDir)).add(aspectName);
 				}
-				((List) outputDirsToAspects.get(outputDir)).add(aspectName);
 			}
 		}
 		return outputDirsToAspects;
@@ -786,13 +787,13 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		}
 		model.setRoot(new ProgramElement(structureModel, rootLabel, kind, new ArrayList()));
 
-		model.setFileMap(new HashMap());
+		model.setFileMap(new HashMap<String, IProgramElement>());
 		// setStructureModel(model);
 		state.setStructureModel(structureModel);
 		// state.setRelationshipMap(AsmManager.getDefault().getRelationshipMap());
 	}
 
-	//    
+	//
 	// private void dumplist(List l) {
 	// System.err.println("---- "+l.size());
 	// for (int i =0 ;i<l.size();i++) System.err.println(i+"\t "+l.get(i));
@@ -873,18 +874,16 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 			outputDir = buildConfig.getCompilationResultDestinationManager().getDefaultOutputLocation();
 		}
 		// ??? incremental issues
-		for (Iterator i = buildConfig.getInJars().iterator(); i.hasNext();) {
-			File inJar = (File) i.next();
-			List unwovenClasses = bcelWeaver.addJarFile(inJar, outputDir, false);
+		for (File inJar : buildConfig.getInJars()) {
+			List<UnwovenClassFile> unwovenClasses = bcelWeaver.addJarFile(inJar, outputDir, false);
 			state.recordBinarySource(inJar.getPath(), unwovenClasses);
 		}
 
-		for (Iterator i = buildConfig.getInpath().iterator(); i.hasNext();) {
-			File inPathElement = (File) i.next();
+		for (File inPathElement : buildConfig.getInpath()) {
 			if (!inPathElement.isDirectory()) {
 				// its a jar file on the inpath
 				// the weaver method can actually handle dirs, but we don't call it, see next block
-				List unwovenClasses = bcelWeaver.addJarFile(inPathElement, outputDir, true);
+				List<UnwovenClassFile> unwovenClasses = bcelWeaver.addJarFile(inPathElement, outputDir, true);
 				state.recordBinarySource(inPathElement.getPath(), unwovenClasses);
 			} else {
 				// add each class file in an in-dir individually, this gives us the best error reporting
@@ -893,7 +892,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 				File[] binSrcs = FileUtil.listFiles(inPathElement, binarySourceFilter);
 				for (int j = 0; j < binSrcs.length; j++) {
 					UnwovenClassFile ucf = bcelWeaver.addClassFile(binSrcs[j], inPathElement, outputDir);
-					List ucfl = new ArrayList();
+					List<UnwovenClassFile> ucfl = new ArrayList<UnwovenClassFile>();
 					ucfl.add(ucf);
 					state.recordBinarySource(binSrcs[j].getPath(), ucfl);
 				}
@@ -973,7 +972,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		return System.getProperty("user.dir"); //$NON-NLS-1$
 	}
 
-	public void performCompilation(Collection files) {
+	public void performCompilation(Collection<File> files) {
 		if (progressListener != null) {
 			compiledCount = 0;
 			sourceFileCount = files.size();
@@ -983,8 +982,8 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		// Translate from strings to File objects
 		String[] filenames = new String[files.size()];
 		int idx = 0;
-		for (Iterator fIterator = files.iterator(); fIterator.hasNext();) {
-			File f = (File) fIterator.next();
+		for (Iterator<File> fIterator = files.iterator(); fIterator.hasNext();) {
+			File f = fIterator.next();
 			filenames[idx++] = f.getPath();
 		}
 
@@ -998,11 +997,11 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		}
 
 		if (environment == null || environmentNeedsRebuilding) {
-			List cps = buildConfig.getFullClasspath();
+			List<String> cps = buildConfig.getFullClasspath();
 			Dump.saveFullClasspath(cps);
 			String[] classpaths = new String[cps.size()];
 			for (int i = 0; i < cps.size(); i++) {
-				classpaths[i] = (String) cps.get(i);
+				classpaths[i] = cps.get(i);
 			}
 			environment = new StatefulNameEnvironment(getLibraryAccess(classpaths, filenames), state.getClassNameToFileMap(), state);
 			state.setNameEnvironment(environment);
@@ -1066,10 +1065,10 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 				// this is either a jar file or a file in a directory
 				boolean hasErrors = unitResult.hasErrors();
 				if (!hasErrors || proceedOnError()) {
-					Collection classFiles = unitResult.compiledTypes.values();
+					Collection<ClassFile> classFiles = unitResult.compiledTypes.values();
 					boolean shouldAddAspectName = (buildConfig.getOutxmlName() != null);
-					for (Iterator iter = classFiles.iterator(); iter.hasNext();) {
-						ClassFile classFile = (ClassFile) iter.next();
+					for (Iterator<ClassFile> iter = classFiles.iterator(); iter.hasNext();) {
+						ClassFile classFile = iter.next();
 						String filename = new String(classFile.fileName());
 						String classname = filename.replace('/', '.');
 						filename = filename.replace('/', File.separatorChar) + ".class";
@@ -1077,6 +1076,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 						try {
 							if (buildConfig.getOutputJar() == null) {
 								String outfile = writeDirectoryEntry(unitResult, classFile, filename);
+								getWorld().classWriteEvent(classFile.getCompoundName());
 								if (environmentSupportsIncrementalCompilation) {
 									if (!classname.endsWith("$ajcMightHaveAspect")) {
 										ResolvedType type = getBcelWorld().resolve(classname);
@@ -1185,7 +1185,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	// ClassFile classFile = (ClassFile) classFiles.nextElement();
 	// String filename = new String(classFile.fileName());
 	// filename = filename.replace('/', File.separatorChar) + ".class";
-	//				
+	//
 	// File destinationPath = buildConfig.getOutputDir();
 	// if (destinationPath == null) {
 	// filename = new File(filename).getName();
@@ -1193,7 +1193,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	// } else {
 	// filename = new File(destinationPath, filename).getPath();
 	// }
-	//				
+	//
 	// //System.out.println("classfile: " + filename);
 	// unwovenClassFiles.add(new UnwovenClassFile(filename, classFile.getBytes()));
 	// }
@@ -1210,7 +1210,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	// state.noteClassesFromFile(null, sourceFileName, Collections.EMPTY_LIST);
 	// }
 	// }
-	//    
+	//
 
 	private void setBuildConfig(AjBuildConfig buildConfig) {
 		this.buildConfig = buildConfig;
