@@ -1,6 +1,5 @@
 /* *******************************************************************
- * Copyright (c) 2002 Palo Alto Research Center, Incorporated (PARC).
- *               2005 Contributors
+ * Copyright (c) 2002,2010
  * All rights reserved. 
  * This program and the accompanying materials are made available 
  * under the terms of the Eclipse Public License v1.0 
@@ -9,7 +8,8 @@
  *  
  * Contributors: 
  *     PARC     initial implementation
- *     Adrian Colyer many updates since.... 
+ *     Adrian Colyer, IBM
+ *     Andy Clement, IBM, SpringSource
  * ******************************************************************/
 
 package org.aspectj.weaver.patterns;
@@ -31,7 +31,12 @@ import org.aspectj.weaver.internal.tools.PointcutDesignatorHandlerBasedPointcut;
 import org.aspectj.weaver.tools.ContextBasedMatcher;
 import org.aspectj.weaver.tools.PointcutDesignatorHandler;
 
-//XXX doesn't handle errors for extra tokens very well (sometimes ignores)
+/**
+ * @author PARC
+ * @author Adrian Colyer
+ * @author Andy Clement
+ */
+// XXX doesn't handle errors for extra tokens very well (sometimes ignores)
 public class PatternParser {
 
 	private ITokenSource tokenSource;
@@ -140,8 +145,8 @@ public class PatternParser {
 			ret = parseSoft();
 		} else {
 			throw new ParserException(
-					"expected one of error, warning, parents, soft, precedence, @type, @method, @constructor, @field", tokenSource
-							.peek(-1));
+					"expected one of error, warning, parents, soft, precedence, @type, @method, @constructor, @field",
+					tokenSource.peek(-1));
 		}
 		int endPos = tokenSource.peek(-1).getEnd();
 		ret.setLocation(sourceContext, startPos, endPos);
@@ -193,23 +198,39 @@ public class PatternParser {
 
 	public DeclareAnnotation parseDeclareAtField() {
 		ISignaturePattern compoundFieldSignaturePattern = parseCompoundFieldSignaturePattern();
-		return new DeclareAnnotation(DeclareAnnotation.AT_FIELD, compoundFieldSignaturePattern);
+		DeclareAnnotation da = new DeclareAnnotation(DeclareAnnotation.AT_FIELD, compoundFieldSignaturePattern);
+		return da;
 	}
 
 	public ISignaturePattern parseCompoundFieldSignaturePattern() {
-		ISignaturePattern atomicFieldSignaturePattern = parseMaybeParenthesizedFieldSignaturePattern();
+		int index = tokenSource.getIndex();
+		try {
+			ISignaturePattern atomicFieldSignaturePattern = parseMaybeParenthesizedFieldSignaturePattern();
 
-		while (isEitherAndOrOr()) {
-			if (maybeEat("&&")) {
-				atomicFieldSignaturePattern = new AndSignaturePattern(atomicFieldSignaturePattern,
-						parseMaybeParenthesizedFieldSignaturePattern());
+			while (isEitherAndOrOr()) {
+				if (maybeEat("&&")) {
+					atomicFieldSignaturePattern = new AndSignaturePattern(atomicFieldSignaturePattern,
+							parseMaybeParenthesizedFieldSignaturePattern());
+				}
+				if (maybeEat("||")) {
+					atomicFieldSignaturePattern = new OrSignaturePattern(atomicFieldSignaturePattern,
+							parseMaybeParenthesizedFieldSignaturePattern());
+				}
 			}
-			if (maybeEat("||")) {
-				atomicFieldSignaturePattern = new OrSignaturePattern(atomicFieldSignaturePattern,
-						parseMaybeParenthesizedFieldSignaturePattern());
+			return atomicFieldSignaturePattern;
+		} catch (ParserException e) {
+			// fallback in the case of a regular single field signature pattern that just happened to start with '('
+			int nowAt = tokenSource.getIndex();
+			tokenSource.setIndex(index);
+			try {
+				ISignaturePattern fsp = parseFieldSignaturePattern();
+				return fsp;
+			} catch (Exception e2) {
+				tokenSource.setIndex(nowAt);
+				// throw the original
+				throw e;
 			}
 		}
-		return atomicFieldSignaturePattern;
 	}
 
 	private boolean isEitherAndOrOr() {
@@ -326,7 +347,7 @@ public class PatternParser {
 	private Pointcut parseNotOrPointcut() {
 		Pointcut p = parseAtomicPointcut();
 		if (maybeEat("&&")) {
-			p = new AndPointcut(p, parsePointcut());
+			p = new AndPointcut(p, parseNotOrPointcut());
 		}
 		return p;
 	}
@@ -751,6 +772,7 @@ public class PatternParser {
 			return p;
 		}
 		if (maybeEat("(")) {
+			int openParenPos = tokenSource.peek(-1).getStart();
 			TypePattern p = parseTypePattern(insideTypeParameters, false);
 			if ((p instanceof NotTypePattern) && !(ap instanceof AnyAnnotationTypePattern)) {
 				// dont set the annotation on it, we don't want the annotation to be
@@ -761,6 +783,7 @@ public class PatternParser {
 				p = setAnnotationPatternForTypePattern(p, ap, parameterAnnotationsPossible);
 			}
 			eat(")");
+			int closeParenPos = tokenSource.peek(-1).getStart();
 			boolean isVarArgs = maybeEat("...");
 			if (isVarArgs) {
 				p.setIsVarArgs(isVarArgs);
@@ -769,9 +792,14 @@ public class PatternParser {
 			if (isIncludeSubtypes) {
 				p.includeSubtypes = true; // need the test because (A+) should not set subtypes to false!
 			}
+			p.start = openParenPos;
+			p.end = closeParenPos;
 			return p;
 		}
 		int startPos = tokenSource.peek().getStart();
+		if (ap.start != -1) {
+			startPos = ap.start;
+		}
 		TypePattern p = parseSingleTypePattern(insideTypeParameters);
 		int endPos = tokenSource.peek(-1).getEnd();
 		p = setAnnotationPatternForTypePattern(p, ap, false);
@@ -852,6 +880,7 @@ public class PatternParser {
 				eat(")");
 				return ret;
 			} else {
+				int atPos = tokenSource.peek(-1).getStart();
 				TypePattern p = parseSingleTypePattern();
 				if (maybeEatAdjacent("(")) {
 					values = parseAnnotationValues();
@@ -860,6 +889,7 @@ public class PatternParser {
 				} else {
 					ret = new WildAnnotationTypePattern(p);
 				}
+				ret.start = atPos;
 				return ret;
 			}
 		} else {
@@ -1108,7 +1138,7 @@ public class PatternParser {
 	// if (maybeEat("&&")) {
 	// ap = new AndAnnotationTypePattern(ap, parseNotOrAnnotationPattern());
 	// }
-	//		
+	//
 	// if (maybeEat("||")) {
 	// ap = new OrAnnotationTypePattern(ap, parseAnnotationTypePattern());
 	// }
