@@ -11,6 +11,8 @@
  *******************************************************************************/
 package org.aspectj.weaver.bcel;
 
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -29,6 +31,7 @@ import org.aspectj.apache.bcel.classfile.JavaClass;
 import org.aspectj.apache.bcel.classfile.LocalVariable;
 import org.aspectj.apache.bcel.classfile.LocalVariableTable;
 import org.aspectj.apache.bcel.classfile.Method;
+import org.aspectj.apache.bcel.classfile.Unknown;
 import org.aspectj.apache.bcel.classfile.annotation.AnnotationGen;
 import org.aspectj.apache.bcel.classfile.annotation.ArrayElementValue;
 import org.aspectj.apache.bcel.classfile.annotation.ClassElementValue;
@@ -48,16 +51,19 @@ import org.aspectj.bridge.MessageUtil;
 import org.aspectj.weaver.Advice;
 import org.aspectj.weaver.AdviceKind;
 import org.aspectj.weaver.AjAttribute;
+import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 import org.aspectj.weaver.AjcMemberMaker;
 import org.aspectj.weaver.BindingScope;
 import org.aspectj.weaver.ISourceContext;
 import org.aspectj.weaver.MethodDelegateTypeMunger;
 import org.aspectj.weaver.NameMangler;
 import org.aspectj.weaver.ReferenceType;
+import org.aspectj.weaver.ReferenceTypeDelegate;
 import org.aspectj.weaver.ResolvedMember;
 import org.aspectj.weaver.ResolvedPointcutDefinition;
 import org.aspectj.weaver.ResolvedType;
 import org.aspectj.weaver.UnresolvedType;
+import org.aspectj.weaver.VersionedDataInputStream;
 import org.aspectj.weaver.WeaverMessages;
 import org.aspectj.weaver.World;
 import org.aspectj.weaver.patterns.Bindings;
@@ -156,13 +162,14 @@ public class AtAjAttributes {
 	private static class AjAttributeFieldStruct extends AjAttributeStruct {
 
 		final Field field;
-		final BcelField bField;
+
+		// final BcelField bField;
 
 		public AjAttributeFieldStruct(Field field, BcelField bField, ResolvedType type, ISourceContext sourceContext,
 				IMessageHandler messageHandler) {
 			super(type, sourceContext, messageHandler);
 			this.field = field;
-			this.bField = bField;
+			// this.bField = bField;
 		}
 	}
 
@@ -224,6 +231,7 @@ public class AtAjAttributes {
 		boolean hasAtAspectAnnotation = false;
 		boolean hasAtPrecedenceAnnotation = false;
 
+		WeaverVersionInfo wvinfo = null;
 		for (int i = 0; i < attributes.length; i++) {
 			Attribute attribute = attributes[i];
 			if (acceptAttribute(attribute)) {
@@ -240,6 +248,41 @@ public class AtAjAttributes {
 				break;
 			}
 		}
+		for (int i = attributes.length - 1; i >= 0; i--) {
+			Attribute attribute = attributes[i];
+			if (attribute.getName().equals(WeaverVersionInfo.AttributeName)) {
+				try {
+					VersionedDataInputStream s = new VersionedDataInputStream(new ByteArrayInputStream(
+							((Unknown) attribute).getBytes()), null);
+					wvinfo = WeaverVersionInfo.read(s);
+					struct.ajAttributes.add(0, wvinfo);
+				} catch (IOException ioe) {
+					ioe.printStackTrace();
+				}
+			}
+		}
+		if (wvinfo == null) {
+			// If we are in here due to a resetState() call (presumably because of reweavable state processing), the
+			// original type delegate will have been set with a version but that version will be missing from
+			// the new set of attributes (looks like a bug where the version attribute was not included in the
+			// data compressed into the attribute). So rather than 'defaulting' to current, we should use one
+			// if it set on the delegate for the type.
+			ReferenceTypeDelegate delegate = type.getDelegate();
+			if (delegate instanceof BcelObjectType) {
+				wvinfo = ((BcelObjectType) delegate).getWeaverVersionAttribute();
+				if (wvinfo != null) {
+					if (wvinfo.getMajorVersion() != WeaverVersionInfo.WEAVER_VERSION_MAJOR_UNKNOWN) {
+						// use this one
+						struct.ajAttributes.add(0, wvinfo);
+					} else {
+						wvinfo = null;
+					}
+				}
+			}
+			if (wvinfo == null) {
+				struct.ajAttributes.add(0, wvinfo = new AjAttribute.WeaverVersionInfo());
+			}
+		}
 
 		// basic semantic check
 		if (hasAtPrecedenceAnnotation && !hasAtAspectAnnotation) {
@@ -317,9 +360,6 @@ public class AtAjAttributes {
 				}
 			}
 			if (processedPointcut) {
-				// FIXME asc should check we aren't adding multiple versions...
-				// will do once I get the tests passing again...
-				struct.ajAttributes.add(new AjAttribute.WeaverVersionInfo());
 				struct.ajAttributes.addAll(mstruct.ajAttributes);
 			}
 		}
@@ -413,8 +453,8 @@ public class AtAjAttributes {
 				msgHandler.handleMessage(new Message(WeaverMessages.format(WeaverMessages.RETURNING_FORMAL_NOT_DECLARED_IN_ADVICE,
 						e.getFormalName()), IMessage.ERROR, null, bMethod.getSourceLocation()));
 			} catch (ThrownFormalNotDeclaredInAdviceSignatureException e) {
-				msgHandler.handleMessage(new Message(WeaverMessages.format(WeaverMessages.THROWN_FORMAL_NOT_DECLARED_IN_ADVICE, e
-						.getFormalName()), IMessage.ERROR, null, bMethod.getSourceLocation()));
+				msgHandler.handleMessage(new Message(WeaverMessages.format(WeaverMessages.THROWN_FORMAL_NOT_DECLARED_IN_ADVICE,
+						e.getFormalName()), IMessage.ERROR, null, bMethod.getSourceLocation()));
 			}
 		}
 		hasAtAspectJAnnotation = hasAtAspectJAnnotation || hasAtAspectJAnnotationMustReturnVoid;
@@ -519,8 +559,8 @@ public class AtAjAttributes {
 				perClause.setLocation(struct.context, -1, -1);// struct.context.getOffset(),
 				// struct.context.getOffset()+1);//FIXME
 				// AVASM
-				// FIXME asc see related comment way about about the version...
-				struct.ajAttributes.add(new AjAttribute.WeaverVersionInfo());
+				// Not setting version here
+				// struct.ajAttributes.add(new AjAttribute.WeaverVersionInfo());
 				AjAttribute.Aspect aspectAttribute = new AjAttribute.Aspect(perClause);
 				struct.ajAttributes.add(aspectAttribute);
 				FormalBinding[] bindings = new org.aspectj.weaver.patterns.FormalBinding[0];
@@ -692,7 +732,7 @@ public class AtAjAttributes {
 					FormalBinding[] bindings = new org.aspectj.weaver.patterns.FormalBinding[0];
 					IScope binding = new BindingScope(struct.enclosingType, struct.context, bindings);
 					// first add the declare implements like
-					List parents = new ArrayList(1);
+					List<TypePattern> parents = new ArrayList<TypePattern>(1);
 					parents.add(parent);
 					DeclareParents dp = new DeclareParents(typePattern, parents, false);
 					dp.resolve(binding); // resolves the parent and child parts
@@ -967,12 +1007,12 @@ public class AtAjAttributes {
 
 		// The factory method for building the implementation is the
 		// one attached to the annotation:
-		Method implementationFactory = struct.method;
+		// Method implementationFactory = struct.method;
 
 		boolean hasAtLeastOneMethod = false;
 
-		for (Iterator iterator = newInterfaceTypes.iterator(); iterator.hasNext();) {
-			ResolvedType typeForDelegation = (ResolvedType) iterator.next();
+		for (Iterator<ResolvedType> iterator = newInterfaceTypes.iterator(); iterator.hasNext();) {
+			ResolvedType typeForDelegation = iterator.next();
 			// TODO check for overlapping interfaces. Eg. A implements I, I extends J - if they specify interfaces={I,J} we dont
 			// want to do any methods twice
 			ResolvedMember[] methods = typeForDelegation.getMethodsWithoutIterator(true, false, false).toArray(
@@ -1043,8 +1083,8 @@ public class AtAjAttributes {
 				}
 				setIgnoreUnboundBindingNames(pc, bindings);
 
-				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(), struct.bMethod
-						.getDeclarationOffset());
+				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(),
+						struct.bMethod.getDeclarationOffset());
 				struct.ajAttributes.add(new AjAttribute.AdviceAttribute(AdviceKind.Before, pc, extraArgument, sl.getOffset(), sl
 						.getOffset() + 1,// FIXME AVASM
 						struct.context));
@@ -1095,8 +1135,8 @@ public class AtAjAttributes {
 				}
 				setIgnoreUnboundBindingNames(pc, bindings);
 
-				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(), struct.bMethod
-						.getDeclarationOffset());
+				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(),
+						struct.bMethod.getDeclarationOffset());
 				struct.ajAttributes.add(new AjAttribute.AdviceAttribute(AdviceKind.After, pc, extraArgument, sl.getOffset(), sl
 						.getOffset() + 1,// FIXME AVASM
 						struct.context));
@@ -1186,8 +1226,8 @@ public class AtAjAttributes {
 			}
 			setIgnoreUnboundBindingNames(pc, bindings);
 
-			ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(), struct.bMethod
-					.getDeclarationOffset());
+			ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(),
+					struct.bMethod.getDeclarationOffset());
 			struct.ajAttributes.add(new AjAttribute.AdviceAttribute(AdviceKind.AfterReturning, pc, extraArgument, sl.getOffset(),
 					sl.getOffset() + 1,// FIXME AVASM
 					struct.context));
@@ -1276,8 +1316,8 @@ public class AtAjAttributes {
 			}
 			setIgnoreUnboundBindingNames(pc, bindings);
 
-			ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(), struct.bMethod
-					.getDeclarationOffset());
+			ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(),
+					struct.bMethod.getDeclarationOffset());
 			struct.ajAttributes.add(new AjAttribute.AdviceAttribute(AdviceKind.AfterThrowing, pc, extraArgument, sl.getOffset(), sl
 					.getOffset() + 1, struct.context));
 			return true;
@@ -1326,8 +1366,8 @@ public class AtAjAttributes {
 				}
 				setIgnoreUnboundBindingNames(pc, bindings);
 
-				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(), struct.bMethod
-						.getDeclarationOffset());
+				ISourceLocation sl = struct.context.makeSourceLocation(struct.bMethod.getDeclarationLineNumber(),
+						struct.bMethod.getDeclarationOffset());
 				struct.ajAttributes.add(new AjAttribute.AdviceAttribute(AdviceKind.Around, pc, extraArgument, sl.getOffset(), sl
 						.getOffset() + 1,// FIXME AVASM
 						struct.context));
@@ -1648,8 +1688,7 @@ public class AtAjAttributes {
 	 */
 	private static AnnotationGen getAnnotation(RuntimeAnnos rvs, UnresolvedType annotationType) {
 		final String annotationTypeName = annotationType.getName();
-		for (Iterator iterator = rvs.getAnnotations().iterator(); iterator.hasNext();) {
-			AnnotationGen rv = (AnnotationGen) iterator.next();
+		for (AnnotationGen rv : rvs.getAnnotations()) {
 			if (annotationTypeName.equals(rv.getTypeName())) {
 				return rv;
 			}
@@ -1665,8 +1704,7 @@ public class AtAjAttributes {
 	 * @return annotation NVP
 	 */
 	private static NameValuePair getAnnotationElement(AnnotationGen annotation, String elementName) {
-		for (Iterator iterator1 = annotation.getValues().iterator(); iterator1.hasNext();) {
-			NameValuePair element = (NameValuePair) iterator1.next();
+		for (NameValuePair element : annotation.getValues()) {
 			if (elementName.equals(element.getNameString())) {
 				return element;
 			}
@@ -1778,10 +1816,8 @@ public class AtAjAttributes {
 		}
 
 		// sort by index
-		Collections.sort(arguments, new Comparator() {
-			public int compare(Object o, Object o1) {
-				MethodArgument mo = (MethodArgument) o;
-				MethodArgument mo1 = (MethodArgument) o1;
+		Collections.sort(arguments, new Comparator<MethodArgument>() {
+			public int compare(MethodArgument mo, MethodArgument mo1) {
 				if (mo.indexOnStack == mo1.indexOnStack) {
 					return 0;
 				} else if (mo.indexOnStack > mo1.indexOnStack) {
@@ -1866,14 +1902,14 @@ public class AtAjAttributes {
 	private static void setIgnoreUnboundBindingNames(Pointcut pointcut, FormalBinding[] bindings) {
 		// register ImplicitBindings as to be ignored since unbound
 		// TODO is it likely to fail in a bad way if f.e. this(jp) etc ?
-		List ignores = new ArrayList();
+		List<String> ignores = new ArrayList<String>();
 		for (int i = 0; i < bindings.length; i++) {
 			FormalBinding formalBinding = bindings[i];
 			if (formalBinding instanceof FormalBinding.ImplicitFormalBinding) {
 				ignores.add(formalBinding.getName());
 			}
 		}
-		pointcut.m_ignoreUnboundBindingForNames = (String[]) ignores.toArray(new String[ignores.size()]);
+		pointcut.m_ignoreUnboundBindingForNames = ignores.toArray(new String[ignores.size()]);
 	}
 
 	/**
@@ -1991,4 +2027,4 @@ public class AtAjAttributes {
 			return formalName;
 		}
 	}
-}
+}
\ No newline at end of file
