@@ -28,16 +28,19 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.jar.Attributes;
+import java.util.jar.Attributes.Name;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
-import java.util.jar.Attributes.Name;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
 import java.util.zip.ZipOutputStream;
 
 import org.aspectj.apache.bcel.classfile.ClassParser;
 import org.aspectj.apache.bcel.classfile.JavaClass;
+import org.aspectj.asm.AsmManager;
+import org.aspectj.asm.IProgramElement;
+import org.aspectj.asm.internal.AspectJElementHierarchy;
 import org.aspectj.bridge.IMessage;
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.Message;
@@ -50,6 +53,7 @@ import org.aspectj.util.FileUtil;
 import org.aspectj.util.FuzzyBoolean;
 import org.aspectj.weaver.Advice;
 import org.aspectj.weaver.AdviceKind;
+import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 import org.aspectj.weaver.AnnotationAJ;
 import org.aspectj.weaver.AnnotationOnTypeMunger;
 import org.aspectj.weaver.BCException;
@@ -115,8 +119,8 @@ public class BcelWeaver {
 	// These four are setup by prepareForWeave
 	private transient List<ShadowMunger> shadowMungerList = null;
 	private transient List<ConcreteTypeMunger> typeMungerList = null;
-	private transient List lateTypeMungerList = null;
-	private transient List declareParentsList = null;
+	private transient List<ConcreteTypeMunger> lateTypeMungerList = null;
+	private transient List<DeclareParents> declareParentsList = null;
 
 	private Manifest manifest = null;
 	private boolean needToReweaveWorld = false;
@@ -494,8 +498,6 @@ public class BcelWeaver {
 			UnwovenClassFile jc = i.next();
 			String name = jc.getClassName();
 			ResolvedType type = world.resolve(name);
-			// System.err.println("added: " + type + " aspect? " +
-			// type.isAspect());
 			if (type.isAspect() && !world.isOverWeaving()) {
 				needToReweaveWorld |= xcutSet.addOrReplaceAspect(type);
 			}
@@ -537,10 +539,8 @@ public class BcelWeaver {
 		// this code may need
 		// a bit of alteration...
 
-		Collections.sort(shadowMungerList, new Comparator() {
-			public int compare(Object o1, Object o2) {
-				ShadowMunger sm1 = (ShadowMunger) o1;
-				ShadowMunger sm2 = (ShadowMunger) o2;
+		Collections.sort(shadowMungerList, new Comparator<ShadowMunger>() {
+			public int compare(ShadowMunger sm1, ShadowMunger sm2) {
 				if (sm1.getSourceLocation() == null) {
 					return (sm2.getSourceLocation() == null ? 0 : 1);
 				}
@@ -928,8 +928,8 @@ public class BcelWeaver {
 	 * @param userPointcut
 	 */
 	private void raiseUnboundFormalError(String name, Pointcut userPointcut) {
-		world.showMessage(IMessage.ERROR, WeaverMessages.format(WeaverMessages.UNBOUND_FORMAL, name), userPointcut
-				.getSourceLocation(), null);
+		world.showMessage(IMessage.ERROR, WeaverMessages.format(WeaverMessages.UNBOUND_FORMAL, name),
+				userPointcut.getSourceLocation(), null);
 	}
 
 	public void addManifest(Manifest newManifest) {
@@ -1004,6 +1004,8 @@ public class BcelWeaver {
 		return c;
 	}
 
+	private Set<IProgramElement> candidatesForRemoval = null;
+
 	// variation of "weave" that sources class files from an external source.
 	public Collection<String> weave(IClassFileProvider input) throws IOException {
 		if (trace.isTraceEnabled()) {
@@ -1013,11 +1015,16 @@ public class BcelWeaver {
 		Collection<String> wovenClassNames = new ArrayList<String>();
 		IWeaveRequestor requestor = input.getRequestor();
 
-		for (Iterator<UnwovenClassFile> i = input.getClassFileIterator(); i.hasNext();) {
-			UnwovenClassFile classFile = i.next();
-			if (world.getModel() != null /* AsmManager.isCreatingModel() */&& !isBatchWeave) {
-				// remove all relationships where this file being woven is the target of the relationship
-				world.getModelAsAsmManager().removeRelationshipsTargettingThisType(classFile.getClassName());
+		if (world.getModel() != null && world.isMinimalModel()) {
+			candidatesForRemoval = new HashSet<IProgramElement>();
+		}
+		if (world.getModel() != null && !isBatchWeave) {
+			AsmManager manager = world.getModelAsAsmManager();
+			for (Iterator<UnwovenClassFile> i = input.getClassFileIterator(); i.hasNext();) {
+				UnwovenClassFile classFile = i.next();
+				// remove all relationships where this file being woven is
+				// the target of the relationship
+				manager.removeRelationshipsTargettingThisType(classFile.getClassName());
 			}
 		}
 
@@ -1184,6 +1191,9 @@ public class BcelWeaver {
 		if (trace.isTraceEnabled()) {
 			trace.exit("weave", wovenClassNames);
 		}
+		if (world.getModel() != null && world.isMinimalModel()) {
+			candidatesForRemoval.clear();
+		}
 		return wovenClassNames;
 	}
 
@@ -1286,8 +1296,11 @@ public class BcelWeaver {
 		ResolvedType superclassType = resolvedTypeToWeave.getSuperclass();
 		String superclassTypename = (superclassType == null ? null : superclassType.getName());
 
-		if (superclassType != null && !superclassType.isTypeHierarchyComplete() && superclassType.isExposedToWeaver()) { // typesForWeaving.contains(superclassTypename))
-			// // {
+		// PR336654 added the 'typesForWeaving.contains(superclassTypename)' clause.
+		// Without it we can delete all type mungers on the parents and yet we only
+		// add back in the declare parents related ones, not the regular ITDs.
+		if (superclassType != null && !superclassType.isTypeHierarchyComplete() && superclassType.isExposedToWeaver()
+				&& typesForWeaving.contains(superclassTypename)) {
 			weaveParentsFor(typesForWeaving, superclassTypename, superclassType);
 		}
 
@@ -1331,8 +1344,8 @@ public class BcelWeaver {
 					ResolvedType rtx = world.resolve(UnresolvedType.forSignature(requiredTypeSignature), true);
 					boolean exists = !rtx.isMissing();
 					if (!exists) {
-						world.getLint().missingAspectForReweaving.signal(new String[] { rtx.getName(), className }, classType
-								.getSourceLocation(), null);
+						world.getLint().missingAspectForReweaving.signal(new String[] { rtx.getName(), className },
+								classType.getSourceLocation(), null);
 						// world.showMessage(IMessage.ERROR, WeaverMessages.format(WeaverMessages.MISSING_REWEAVABLE_TYPE,
 						// requiredTypeName, className), classType.getSourceLocation(), null);
 					} else {
@@ -1349,8 +1362,8 @@ public class BcelWeaver {
 								world.showMessage(IMessage.ERROR, WeaverMessages.format(
 										WeaverMessages.REWEAVABLE_ASPECT_NOT_REGISTERED, rtx.getName(), className), null, null);
 							} else if (!world.getMessageHandler().isIgnoring(IMessage.INFO)) {
-								world.showMessage(IMessage.INFO, WeaverMessages.format(WeaverMessages.VERIFIED_REWEAVABLE_TYPE, rtx
-										.getName(), rtx.getSourceLocation().getSourceFile()), null, null);
+								world.showMessage(IMessage.INFO, WeaverMessages.format(WeaverMessages.VERIFIED_REWEAVABLE_TYPE,
+										rtx.getName(), rtx.getSourceLocation().getSourceFile()), null, null);
 							}
 						}
 						alreadyConfirmedReweavableState.add(requiredTypeSignature);
@@ -1363,7 +1376,9 @@ public class BcelWeaver {
 			// new: reweavable default with clever diff
 			if (!world.isOverWeaving()) {
 				byte[] bytes = wsi.getUnwovenClassFileData(classType.getJavaClass().getBytes());
-				classType.setJavaClass(Utility.makeJavaClass(classType.getJavaClass().getFileName(), bytes), true);
+				WeaverVersionInfo wvi = classType.getWeaverVersionAttribute();
+				JavaClass newJavaClass = Utility.makeJavaClass(classType.getJavaClass().getFileName(), bytes);
+				classType.setJavaClass(newJavaClass, true);
 				classType.getResolvedTypeX().ensureConsistent();
 			}
 			// } else {
@@ -1416,12 +1431,12 @@ public class BcelWeaver {
 	}
 
 	public UnwovenClassFile[] getClassFilesFor(LazyClassGen clazz) {
-		List childClasses = clazz.getChildClasses(world);
+		List<UnwovenClassFile.ChildClass> childClasses = clazz.getChildClasses(world);
 		UnwovenClassFile[] ret = new UnwovenClassFile[1 + childClasses.size()];
 		ret[0] = new UnwovenClassFile(clazz.getFileName(), clazz.getClassName(), clazz.getJavaClassBytesIncludingReweavable(world));
 		int index = 1;
-		for (Iterator iter = childClasses.iterator(); iter.hasNext();) {
-			UnwovenClassFile.ChildClass element = (UnwovenClassFile.ChildClass) iter.next();
+		for (Iterator<UnwovenClassFile.ChildClass> iter = childClasses.iterator(); iter.hasNext();) {
+			UnwovenClassFile.ChildClass element = iter.next();
 			UnwovenClassFile childClass = new UnwovenClassFile(clazz.getFileName() + "$" + element.name, element.bytes);
 			ret[index++] = childClass;
 		}
@@ -1447,8 +1462,7 @@ public class BcelWeaver {
 		boolean aParentChangeOccurred = false;
 		boolean anAnnotationChangeOccurred = false;
 		// First pass - apply all decp mungers
-		for (Iterator i = declareParentsList.iterator(); i.hasNext();) {
-			DeclareParents decp = (DeclareParents) i.next();
+		for (DeclareParents decp : declareParentsList) {
 			boolean typeChanged = applyDeclareParents(decp, onType);
 			if (typeChanged) {
 				aParentChangeOccurred = true;
@@ -1458,8 +1472,7 @@ public class BcelWeaver {
 		}
 
 		// Still first pass - apply all dec @type mungers
-		for (Iterator i = xcutSet.getDeclareAnnotationOnTypes().iterator(); i.hasNext();) {
-			DeclareAnnotation decA = (DeclareAnnotation) i.next();
+		for (DeclareAnnotation decA : xcutSet.getDeclareAnnotationOnTypes()) {
 			boolean typeChanged = applyDeclareAtType(decA, onType, true);
 			if (typeChanged) {
 				anAnnotationChangeOccurred = true;
@@ -1479,8 +1492,7 @@ public class BcelWeaver {
 				}
 			}
 
-			for (Iterator iter = xcutSet.getDeclareAnnotationOnTypes().iterator(); iter.hasNext();) {
-				DeclareAnnotation decA = (DeclareAnnotation) iter.next();
+			for (DeclareAnnotation decA : xcutSet.getDeclareAnnotationOnTypes()) {
 				boolean typeChanged = applyDeclareAtType(decA, onType, false);
 				if (typeChanged) {
 					anAnnotationChangeOccurred = true;
@@ -1521,14 +1533,15 @@ public class BcelWeaver {
 
 			if (!problemReported) {
 				AsmRelationshipProvider.addDeclareAnnotationRelationship(world.getModelAsAsmManager(), decA.getSourceLocation(),
-						onType.getSourceLocation());
+						onType.getSourceLocation(), false);
 				// TAG: WeavingMessage
 				if (!getWorld().getMessageHandler().isIgnoring(IMessage.WEAVEINFO)) {
 					getWorld().getMessageHandler().handleMessage(
-							WeaveMessage.constructWeavingMessage(WeaveMessage.WEAVEMESSAGE_ANNOTATES, new String[] {
-									onType.toString(), Utility.beautifyLocation(onType.getSourceLocation()),
-									decA.getAnnotationString(), "type", decA.getAspect().toString(),
-									Utility.beautifyLocation(decA.getSourceLocation()) }));
+							WeaveMessage.constructWeavingMessage(
+									WeaveMessage.WEAVEMESSAGE_ANNOTATES,
+									new String[] { onType.toString(), Utility.beautifyLocation(onType.getSourceLocation()),
+											decA.getAnnotationString(), "type", decA.getAspect().toString(),
+											Utility.beautifyLocation(decA.getSourceLocation()) }));
 				}
 				didSomething = true;
 				ResolvedTypeMunger newAnnotationTM = new AnnotationOnTypeMunger(annoX);
@@ -1551,13 +1564,15 @@ public class BcelWeaver {
 				if (outputProblems) {
 					if (decA.isExactPattern()) {
 						world.getMessageHandler().handleMessage(
-								MessageUtil.error(WeaverMessages.format(WeaverMessages.INCORRECT_TARGET_FOR_DECLARE_ANNOTATION,
-										onType.getName(), annoX.getTypeName(), annoX.getValidTargets()), decA.getSourceLocation()));
+								MessageUtil.error(
+										WeaverMessages.format(WeaverMessages.INCORRECT_TARGET_FOR_DECLARE_ANNOTATION,
+												onType.getName(), annoX.getTypeName(), annoX.getValidTargets()),
+										decA.getSourceLocation()));
 					} else {
 						if (world.getLint().invalidTargetForAnnotation.isEnabled()) {
 							world.getLint().invalidTargetForAnnotation.signal(new String[] { onType.getName(), annoX.getTypeName(),
-									annoX.getValidTargets() }, decA.getSourceLocation(), new ISourceLocation[] { onType
-									.getSourceLocation() });
+									annoX.getValidTargets() }, decA.getSourceLocation(),
+									new ISourceLocation[] { onType.getSourceLocation() });
 						}
 					}
 				}
@@ -1572,14 +1587,12 @@ public class BcelWeaver {
 	 */
 	private boolean applyDeclareParents(DeclareParents p, ResolvedType onType) {
 		boolean didSomething = false;
-		List newParents = p.findMatchingNewParents(onType, true);
+		List<ResolvedType> newParents = p.findMatchingNewParents(onType, true);
 		if (!newParents.isEmpty()) {
 			didSomething = true;
 			BcelWorld.getBcelObjectType(onType);
 			// System.err.println("need to do declare parents for: " + onType);
-			for (Iterator j = newParents.iterator(); j.hasNext();) {
-				ResolvedType newParent = (ResolvedType) j.next();
-
+			for (ResolvedType newParent : newParents) {
 				// We set it here so that the imminent matching for ITDs can
 				// succeed - we
 				// still haven't done the necessary changes to the class file
@@ -1588,7 +1601,7 @@ public class BcelWeaver {
 				// BcelTypeMunger.mungeNewParent()
 				// classType.addParent(newParent);
 				onType.addParent(newParent);
-				NewParentTypeMunger newParentMunger = new NewParentTypeMunger(newParent);
+				NewParentTypeMunger newParentMunger = new NewParentTypeMunger(newParent, p.getDeclaringType());
 				if (p.isMixin()) {
 					newParentMunger.setIsMixin(true);
 				}
@@ -1600,8 +1613,8 @@ public class BcelWeaver {
 	}
 
 	public void weaveNormalTypeMungers(ResolvedType onType) {
-		ContextToken tok = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.PROCESSING_TYPE_MUNGERS, onType
-				.getName());
+		ContextToken tok = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.PROCESSING_TYPE_MUNGERS,
+				onType.getName());
 		if (onType.isRawType() || onType.isParameterizedType()) {
 			onType = onType.getGenericType();
 		}
@@ -1712,19 +1725,45 @@ public class BcelWeaver {
 				checkDeclareTypeErrorOrWarning(world, classType);
 			}
 			// this is very odd return behavior trying to keep everyone happy
-			/*
-			 * // can we remove it from the model now? we know it contains no relationship endpoints... AsmManager asm =
-			 * world.getModelAsAsmManager(); if (asm != null) { IHierarchy model = asm.getHierarchy(); if (!classType.isAspect()) {
-			 * 
-			 * String pkgname = classType.getResolvedTypeX().getPackageName(); String tname =
-			 * classType.getResolvedTypeX().getSimpleBaseName(); IProgramElement typeElement = model.findElementForType(pkgname,
-			 * tname); if (typeElement != null) { Set deleted = new HashSet();
-			 * deleted.add(asm.getCanonicalFilePath(typeElement.getSourceLocation().getSourceFile()));
-			 * 
-			 * model.updateHandleMap(deleted); IProgramElement parent = typeElement.getParent(); // parent may have children:
-			 * PACKAGE DECL, IMPORT-REFEFERENCE, TYPE_DECL if (parent != null) { typeElement.getParent().removeChild(typeElement);
-			 * // System.out.println("Removing " + classType.getResolvedTypeX().getName() + "? " // + ); } } } }
-			 */
+
+			// can we remove it from the model now? we know it contains no relationship endpoints...
+			AsmManager model = world.getModelAsAsmManager();
+			if (world.isMinimalModel() && model != null && !classType.isAspect()) {
+				AspectJElementHierarchy hierarchy = (AspectJElementHierarchy) model.getHierarchy();
+				String pkgname = classType.getResolvedTypeX().getPackageName();
+				String tname = classType.getResolvedTypeX().getSimpleBaseName();
+				IProgramElement typeElement = hierarchy.findElementForType(pkgname, tname);
+				if (typeElement != null && hasInnerType(typeElement)) {
+					// Cannot remove it right now (has inner type), schedule it
+					// for possible deletion later if all inner types are
+					// removed
+					candidatesForRemoval.add(typeElement);
+				}
+				if (typeElement != null && !hasInnerType(typeElement)) {
+					IProgramElement parent = typeElement.getParent();
+					// parent may have children: PACKAGE DECL, IMPORT-REFERENCE, TYPE_DECL
+					if (parent != null) {
+						// if it was the only type we should probably remove
+						// the others too.
+						parent.removeChild(typeElement);
+						if (parent.getKind().isSourceFile()) {
+							removeSourceFileIfNoMoreTypeDeclarationsInside(hierarchy, typeElement, parent);
+						} else {
+							hierarchy.forget(null, typeElement);
+							// At this point, the child has been removed. We
+							// should now check if the parent is in our
+							// 'candidatesForRemoval' set. If it is then that
+							// means we were going to remove it but it had a
+							// child. Now we can check if it still has a child -
+							// if it doesn't it can also be removed!
+
+							walkUpRemovingEmptyTypesAndPossiblyEmptySourceFile(hierarchy, tname, parent);
+						}
+
+					}
+				}
+			}
+
 			if (dump) {
 				dumpUnchanged(classFile);
 				return clazz;
@@ -1745,8 +1784,86 @@ public class BcelWeaver {
 		}
 	}
 
+	private void walkUpRemovingEmptyTypesAndPossiblyEmptySourceFile(AspectJElementHierarchy hierarchy, String tname,
+			IProgramElement typeThatHasChildRemoved) {
+		// typeThatHasChildRemoved might be a source file, type or a method/ctor
+		// - for a method/ctor find the type/sourcefile
+		while (typeThatHasChildRemoved != null
+				&& !(typeThatHasChildRemoved.getKind().isType() || typeThatHasChildRemoved.getKind().isSourceFile())) {
+			// this will take us 'up' through methods that contain anonymous
+			// inner classes
+			typeThatHasChildRemoved = typeThatHasChildRemoved.getParent();
+		}
+		// now typeThatHasChildRemoved points to the type or sourcefile that has
+		// had something removed
+		if (candidatesForRemoval.contains(typeThatHasChildRemoved) && !hasInnerType(typeThatHasChildRemoved)) {
+			// now we can get rid of it
+			IProgramElement parent = typeThatHasChildRemoved.getParent();
+			if (parent != null) {
+				parent.removeChild(typeThatHasChildRemoved);
+				candidatesForRemoval.remove(typeThatHasChildRemoved);
+				if (parent.getKind().isSourceFile()) {
+					removeSourceFileIfNoMoreTypeDeclarationsInside(hierarchy, typeThatHasChildRemoved, parent);
+					// System.out.println("Removed on second pass: " +
+					// typeThatHasChildRemoved.getName());
+				} else {
+					// System.out.println("On later pass, parent of type " +
+					// typeThatHasChildRemoved.getName()
+					// + " was found not to be a sourcefile, recursing up...");
+					walkUpRemovingEmptyTypesAndPossiblyEmptySourceFile(hierarchy, tname, parent);
+				}
+			}
+		}
+	}
+
+	private void removeSourceFileIfNoMoreTypeDeclarationsInside(AspectJElementHierarchy hierarchy, IProgramElement typeElement,
+			IProgramElement sourceFileNode) {
+		IProgramElement compilationUnit = sourceFileNode;
+		boolean anyOtherTypeDeclarations = false;
+		for (IProgramElement child : compilationUnit.getChildren()) {
+			IProgramElement.Kind k = child.getKind();
+			if (k.isType()) {
+				anyOtherTypeDeclarations = true;
+				break;
+			}
+		}
+		// If the compilation unit node contained no
+		// other types, there is no need to keep it
+		if (!anyOtherTypeDeclarations) {
+			IProgramElement cuParent = compilationUnit.getParent();
+			if (cuParent != null) {
+				compilationUnit.setParent(null);
+				cuParent.removeChild(compilationUnit);
+			}
+			// need to update some caches and structures too?
+			hierarchy.forget(sourceFileNode, typeElement);
+		} else {
+			hierarchy.forget(null, typeElement);
+		}
+	}
+
 	// ---- writing
 
+	// TODO could be smarter - really only matters if inner type has been woven, but there is a chance we haven't woven it *yet*
+	private boolean hasInnerType(IProgramElement typeNode) {
+		for (IProgramElement child : typeNode.getChildren()) {
+			IProgramElement.Kind kind = child.getKind();
+			if (kind.isType()) {
+				return true;
+			}
+			// if (kind == IProgramElement.Kind.ASPECT) {
+			// return true;
+			// }
+			if (kind.isType() || kind == IProgramElement.Kind.METHOD || kind == IProgramElement.Kind.CONSTRUCTOR) {
+				boolean b = hasInnerType(child);
+				if (b) {
+					return b;
+				}
+			}
+		}
+		return false;
+	}
+
 	private void checkDeclareTypeErrorOrWarning(BcelWorld world2, BcelObjectType classType) {
 		List<DeclareTypeErrorOrWarning> dteows = world.getDeclareTypeEows();
 		for (DeclareTypeErrorOrWarning dteow : dteows) {
@@ -1779,9 +1896,10 @@ public class BcelWeaver {
 		if (zipOutputStream != null) {
 			String mainClassName = classFile.getJavaClass().getClassName();
 			writeZipEntry(getEntryName(mainClassName), clazz.getJavaClass(world).getBytes());
-			if (!clazz.getChildClasses(world).isEmpty()) {
-				for (Iterator i = clazz.getChildClasses(world).iterator(); i.hasNext();) {
-					UnwovenClassFile.ChildClass c = (UnwovenClassFile.ChildClass) i.next();
+			List<UnwovenClassFile.ChildClass> childClasses = clazz.getChildClasses(world);
+			if (!childClasses.isEmpty()) {
+				for (Iterator<UnwovenClassFile.ChildClass> i = childClasses.iterator(); i.hasNext();) {
+					UnwovenClassFile.ChildClass c = i.next();
 					writeZipEntry(getEntryName(mainClassName + "$" + c.name), c.bytes);
 				}
 			}
